﻿// Copyright (c) Microsoft. All rights reserved.

using Microsoft.VisualStudio.TestTools.UITest.Extension;
using Microsoft.VisualStudio.TestTools.UITesting;
using Microsoft.VisualStudio.TestTools.UITesting.WindowsRuntimeControls;
using System;
using System.Collections.Generic;

namespace DatePickerExtension_Namespace
{
    public class DatePicker_PropertyProvider : UITestPropertyProvider
    {
        private static UITestPropertyProvider m_corePropProv = null;
        public static UITestPropertyProvider CorePropertyProvider
        {
            get
            {
                if(m_corePropProv == null)
                {
                    m_corePropProv = Playback.GetCorePropertyProvider(new XamlControl());
                }
                return m_corePropProv;
            }
        }

        #region Overriden methods

        public override int GetControlSupportLevel(UITestControl uiTestControl)
        {
            if (uiTestControl.TechnologyName == Utilities.MyTechnologyName && 
                uiTestControl.ControlType == ControlType.Group)
            {
                return (int)ControlSupport.ControlSpecificSupport;
            }
            return (int)ControlSupport.NoSupport;
        }

        /// <summary>
        /// Gets the search properties present by default in the specified specialized class.
        /// </summary>
        /// <param name="specializedClassName">System.Type object of the specialized class.</param>
        /// <returns>Properties already present in the specialized class.</returns>
        public override string[] GetPredefinedSearchProperties(Type specializedClass)
        {
            return CorePropertyProvider.GetPredefinedSearchProperties(specializedClass);
        }

        /// <summary>
        /// Gets the property descriptor for the given property of the control.
        /// </summary>
        /// <param name="uiTestControl">The control for which descriptor is required.</param>
        /// <param name="propertyName">The name of the property for which descriptor is required.</param>
        /// <returns>The property descriptor of the property.</returns>
        public override UITestPropertyDescriptor GetPropertyDescriptor(UITestControl uiTestControl, string propertyName)
        {
            var map = GetAllPropertiesMap(uiTestControl);
            if (map != null && map.ContainsKey(propertyName))
            {
                return map[propertyName];
            }
            return CorePropertyProvider.GetPropertyDescriptor(uiTestControl, propertyName);
        }

        /// <summary>
        /// Gets the property name for the specified action.
        /// For example, to generate code for a SetValueAction on a Edit like: -
        ///         myEdit.Text = "abc";
        /// this method should return "Text" as property name. Otherwise, if this
        /// returns null, then the generated code will look like -
        ///         myEdit.SetProperty("Value", "abc");
        /// </summary>
        /// <param name="uiTestControl">The control on which the action was performed.</param>
        /// <param name="action">The action for which the property is required.</param>
        /// <returns>The writable property name for the action or null if no property exists.</returns>
        public override string GetPropertyForAction(UITestControl uiTestControl, Microsoft.VisualStudio.TestTools.UITest.Common.UITestAction action)
        {
            return CorePropertyProvider.GetPropertyForAction(uiTestControl, action);
        }

        /// <summary>
        /// Gets the property name for the specified control state.
        /// For example, to generate code for a SetStateAction on a TreeItem like: -
        ///         myTreeItem.Expanded = true;
        /// this method should return "Expanded" as property name and true as the stateValue.
        /// Otherwise, if this returns null, then the generated code will look like -
        ///         myEdit.SetProperty("State", ControlStates.Expanded);
        /// </summary>
        /// <param name="uiTestControl">The control for which the state property name is required.</param>
        /// <param name="uiState">The state for which the property names are required.</param>
        /// <param name="stateValues">The values for the properties returned.</param>
        /// <returns>The writable property names for the state or null if no property exists.</returns>
        public override string[] GetPropertyForControlState(UITestControl uiTestControl, Microsoft.VisualStudio.TestTools.UITest.Extension.ControlStates uiState, out bool[] stateValues)
        {
            return CorePropertyProvider.GetPropertyForControlState(uiTestControl, uiState, out stateValues);
        }

        /// <summary>
        /// Gets all properties supported by this provider for the specified control.
        /// </summary>
        /// <param name="uiTestControl">The control whose properties are required.</param>
        /// <returns>The collection of supported properties.</returns>
        public override System.Collections.Generic.ICollection<string> GetPropertyNames(UITestControl uiTestControl)
        {
            if (uiTestControl.TechnologyName == Utilities.MyTechnologyName &&
                uiTestControl.ControlType == ControlType.Group)
                return GetAllPropertiesMap(uiTestControl).Keys;
            return CorePropertyProvider.GetPropertyNames(uiTestControl);
        }

        /// <summary>
        /// Gets the class type of where the constants for the property names are defined.
        /// </summary>
        /// <param name="uiTestControl">The control for which the property defining type is required.</param>
        /// <returns>The class which defines the constants for the property names.</returns>
        public override Type GetPropertyNamesClassType(UITestControl uiTestControl)
        {
            string techName = uiTestControl.TechnologyName;
            if (techName == Utilities.MyTechnologyName &&
                uiTestControl.ControlType == ControlType.Group)
                return typeof(DatePickerControl.PropertyNames);
            return CorePropertyProvider.GetPropertyNamesClassType(uiTestControl);
        }

        /// <summary>
        /// Gets the property value of the control.
        /// This method is called for all the properties defined in UITestControl.PropertyNames
        /// to let the provider override the values. The provider should either support those
        /// properties properly or throw System.NotSupportedException.
        /// </summary>
        /// <param name="uiTestControl">The control for which to get the value.</param>
        /// <param name="propertyName">The name of the property to get.</param>
        /// <returns>The value of the property.</returns>
        public override object GetPropertyValue(UITestControl uiTestControl, string propertyName)
        {
            ControlType controlType = CorePropertyProvider.GetPropertyValue(uiTestControl, UITestControl.PropertyNames.ControlType) as ControlType;
            
            if (uiTestControl.TechnologyName == Utilities.MyTechnologyName &&
                controlType == ControlType.Group &&
                propertyName == "DateTimeValue")
            {
                return GetDateTimeValueProperty(uiTestControl);
            } 

            return CorePropertyProvider.GetPropertyValue(uiTestControl, propertyName);
        }

        /// <summary>
        /// Sets the property value of the control.
        /// This method is called for all the settable properties defined in UITestControl.PropertyNames
        /// to let the provider override the values. The provider should either support setting of those
        /// properties properly or throw System.NotSupportedException.
        /// </summary>
        /// <param name="uiTestControl">The control for which to set the value.</param>
        /// <param name="propertyName">The name of the property to set.</param>
        /// <param name="value">The value of the property.</param>
        public override void SetPropertyValue(UITestControl uiTestControl, string propertyName, object value)
        {
            if (uiTestControl.TechnologyName == Utilities.MyTechnologyName &&
                uiTestControl.ControlType == ControlType.Group &&
                propertyName == "DateTimeValue")
            {
                SetDateTimeValueProperty(uiTestControl, value);
                return;
            } 

            CorePropertyProvider.SetPropertyValue(uiTestControl, propertyName, value);
        }

        /// <summary>
        /// Gets the specialized class to use for the specified control.
        /// </summary>
        /// <param name="uiTestControl">The control for which a specialized class is to needed.</param>
        /// <returns>Type object of the specialized class.</returns>
        public override Type GetSpecializedClass(UITestControl uiTestControl)
        {
            string techName = uiTestControl.TechnologyName;
            if (techName == Utilities.MyTechnologyName &&
                uiTestControl.ControlType == ControlType.Group)
                return typeof(DatePickerControl);
            return CorePropertyProvider.GetSpecializedClass(uiTestControl);
        }

        #endregion

        #region Private Members
        /// <summary>
        /// Dictionary of all the properties.
        /// </summary>
        private static Dictionary<string, UITestPropertyDescriptor> allPropertiesMap;

        /// <summary>
        /// Gets all the properties supported by the control.
        /// </summary>
        /// <param name="control">The control.</param>
        /// <returns>All properties.</returns>
        private static Dictionary<string, UITestPropertyDescriptor> GetAllPropertiesMap(UITestControl uiTestControl)
        {
            if (uiTestControl.TechnologyName == Utilities.MyTechnologyName &&
                uiTestControl.SearchProperties[UITestControl.PropertyNames.ControlType] == ControlType.Group)
            {
                if (allPropertiesMap == null)
                {
                    // Initialize the dictionary.
                    allPropertiesMap = new Dictionary<string, UITestPropertyDescriptor>(StringComparer.OrdinalIgnoreCase);

                    foreach (var property in CorePropertyProvider.GetPropertyNames(uiTestControl))
                    {
                        var descriptor = CorePropertyProvider.GetPropertyDescriptor(uiTestControl, property);
                        allPropertiesMap.Add(property, descriptor);
                    }
                    allPropertiesMap.Add("DateTimeValue", new UITestPropertyDescriptor(typeof(DateTime), UITestPropertyAttributes.Readable | UITestPropertyAttributes.Writable | UITestPropertyAttributes.DoNotGenerateProperties));
                }

                return allPropertiesMap;
            }

            return null;
        }

        private DateTime GetDateTimeValueProperty(UITestControl uiTestControl)
        {
            XamlControl dtp = new XamlControl();
            dtp.CopyFrom(uiTestControl);
            XamlComboBox dayPicker = new XamlComboBox(dtp);
            dayPicker.SearchProperties[XamlComboBox.PropertyNames.AutomationId] = "DayPicker";
            XamlComboBox monthPicker = new XamlComboBox(dtp);
            monthPicker.SearchProperties[XamlComboBox.PropertyNames.AutomationId] = "MonthPicker";
            XamlComboBox yearPicker = new XamlComboBox(dtp);
            yearPicker.SearchProperties[XamlComboBox.PropertyNames.AutomationId] = "YearPicker";

            char a = Convert.ToChar(8206);
            string syear = yearPicker.SelectedItem.Trim(a);
            int year;
            int.TryParse(syear, out year);
            string smonth = monthPicker.SelectedItem.Trim(a);
            int month = MonthToNumberMap[smonth];
            string sday = dayPicker.SelectedItem.Trim(a);
            int day;
            int.TryParse(sday, out day);
            return new DateTime(year, month, day);
        }

        private void SetDateTimeValueProperty(UITestControl uiTestControl, object value)
        {
            DateTime? dateTime = value as DateTime?;
            XamlControl dtp = new XamlControl();
            dtp.CopyFrom(uiTestControl);
            XamlComboBox dayPicker = new XamlComboBox(dtp);
            dayPicker.SearchProperties[XamlComboBox.PropertyNames.AutomationId] = "DayPicker";
            XamlComboBox yearPicker = new XamlComboBox(dtp);
            yearPicker.SearchProperties[XamlComboBox.PropertyNames.AutomationId] = "YearPicker";
            XamlComboBox monthPicker = new XamlComboBox(dtp);
            monthPicker.SearchProperties[XamlComboBox.PropertyNames.AutomationId] = "MonthPicker";
            char a = Convert.ToChar(8206);
            yearPicker.SelectedItem = a + dateTime.Value.Year.ToString();
            monthPicker.SelectedIndex = dateTime.Value.Month - 1;
            dayPicker.SelectedItem = a + dateTime.Value.Day.ToString();
        }

        #endregion


        #region Month Map

        private static Dictionary<string, int> m_MonthToNumberMap = null;
        private static Dictionary<string, int> MonthToNumberMap
        {
            get
            {
                if (m_MonthToNumberMap == null)
                    GenerateMonthMaps();
                return m_MonthToNumberMap;
            }
        }

        private static void GenerateMonthMaps()
        {
            if (m_MonthToNumberMap == null)
                m_MonthToNumberMap = new Dictionary<string, int>();
            GenerateMonthMaps("January", 1);
            GenerateMonthMaps("February", 2);
            GenerateMonthMaps("March", 3);
            GenerateMonthMaps("April", 4);
            GenerateMonthMaps("May", 5);
            GenerateMonthMaps("June", 6);
            GenerateMonthMaps("July", 7);
            GenerateMonthMaps("August", 8);
            GenerateMonthMaps("September", 9);
            GenerateMonthMaps("October", 10);
            GenerateMonthMaps("November", 11);
            GenerateMonthMaps("December", 12);
        }

        private static void GenerateMonthMaps(string month, int number)
        {
            m_MonthToNumberMap[month] = number;
        }

        #endregion
    }
}
