﻿// Copyright (c) Microsoft. All rights reserved.

using Microsoft.VisualStudio.TestTools.UITest.Extension;
using Microsoft.VisualStudio.TestTools.UITest.Input;
using Microsoft.VisualStudio.TestTools.UITesting;
using System;
using System.Runtime.InteropServices;
using System.Windows.Automation;

namespace DatePickerExtension_Namespace
{
    [ComVisible(true)]
    public class DatePicker_TechnologyManager : UITechnologyManagerProxy
    {
        #region ctor
        
        public DatePicker_TechnologyManager()
            : base(Utilities.UiaTechnologyName, Utilities.MyTechnologyName)
        {
        }

        #endregion

        #region overridden methods

        /// <summary>
        /// Conversion of core element to extension element.
        /// All plugins extending this proxy technology manager need to implement this and create a simple conversion with 
        /// copy of core technology element and extension manager. 
        /// <param name="coreElement">The core technology element that needs to be wrapped.</param>
        /// <returns>The extension element.</returns>
        public override IUITechnologyElement ConvertToExtensionElement(IUITechnologyElement coreElement)
        {
            // Return the date picker technology element only if it is a date picker control. Do nothing otherwise.
            if (Microsoft.VisualStudio.TestTools.UITesting.ControlType.Group.NameEquals(coreElement.ControlTypeName))
            {
                return new DatePicker_TechnologyElement(this, coreElement);
            }

            return coreElement;
        }

        /// <summary>
        /// Gets the support level of this technology manager for the element corresponding to the given Automation Element.
        /// The framework uses this function to select the right technology manager for the element. 
        /// </summary>
        /// <param name="element">The automation element corresponding the element.</param>
        /// <returns>An integer that indicates the level of support provided for the element
        /// by this technology manager. The higher the value the stronger the support.</returns>
        /// <seealso cref="ControlSupport"/>
        public override int GetControlSupportLevel(AutomationElement element)
        {
            AutomationElement parent = TreeWalker.ControlViewWalker.GetParent(element);
            // Support only if it is a datepicker control. Do not support the containing comboboxes.
            if(isDatePicker(element))
            {
                return (int)ControlSupport.ControlSpecificSupport;
            }
            return (int)ControlSupport.NoSupport;
        }

        /// <summary>
        /// Gets the element at the given screen coordinates.
        /// </summary>
        /// <param name="pointX">The x-coordinate of the screen location.</param>
        /// <param name="pointY">The y-coordinate of the screen location.</param>
        /// <param name="ceilingElement">The Ceiling element in this technology until which the queryid should be created.</param>
        /// <returns>The IUITechnologyElement at the screen coordinates specified.</returns>
        /// <remarks>The queryid generated for the technology element should not cross the ceilingElement in order to honor other plugins claiming support.</remarks>
        public override IUITechnologyElement GetElementFromPoint(int pointX, int pointY, AutomationElement ceilingElement)
        {
            AutomationElement automationElement = AutomationElement.FromPoint(new Point(pointX, pointY));
            AutomationElement parent = null;
            if (isDatePicker(automationElement))
            {
                IUITechnologyElement element = this.GetElementFromNativeElement(parent);
                return element;
            }
            return base.GetElementFromPoint(pointX, pointY, ceilingElement);
        }

        #endregion

        #region Private Methods

        private bool isDatePicker(AutomationElement ae)
        {
            if (ae == null)
                return false;
            string className = (string)ae.GetCurrentPropertyValue(AutomationElement.ClassNameProperty);
            string frameworkID = (string)ae.GetCurrentPropertyValue(AutomationElement.FrameworkIdProperty);
            if (className != null && className.Equals("DatePicker") && frameworkID != null && frameworkID.Equals("XAML"))
            {
                return true;
            }
            return false;
        }

        #endregion
    }
}
